import os, sys
from subprocess import Popen, PIPE
from time import time

import pytest

inc_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0, inc_path)
import internetarchive


def test_ia_search():
    cmd = 'ia search iacli-test-item --sort=date:asc'
    proc = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE)
    stdout, stderr = proc.communicate()
    assert proc.returncode == 0

    cmd = 'ia search "identifier:iacli-test-item" --number-found'
    proc = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE)
    stdout, stderr = proc.communicate()
    assert stdout == '1\n'
    assert proc.returncode == 0



from collections import defaultdict
import re


# get_xml_text()
#_________________________________________________________________________________________
def get_xml_text(elements, text=''):
    """:todo: document ``get_xml_text()`` function."""
    for e in elements:
        for node in e.childNodes:
            if node.nodeType == node.TEXT_NODE:
                text += node.data
    return text


# get_args_dict()
#_________________________________________________________________________________________
def get_args_dict(args):
    # Convert args list into a metadata dict.
    metadata = defaultdict(list)
    for md in args:
        key, value = md.split(':', 1)
        if value not in metadata[key]:
            metadata[key].append(value)

    #for _key, value in metadata.items():
    for key in metadata:
        # Flatten single item lists.
        if len(metadata[key]) <= 1:
            metadata[key] = metadata[key][0]

    return metadata



"""Search the Internet Archive using the Archive.org Advanced Search
API <https://archive.org/advancedsearch.php#raw>.

usage:
    ia search [--parameters=<key:value>...] [--sort=<field:order>...]
              [--itemlist | --field=<field>...] [--number-found] [--v2] <query>...
    ia search --help

options:
    -h, --help
    -p, --parameters=<key:value>...  Parameters to send with your query.
    -s, --sort=<field:order>...      Sort search results by specified fields.
                                     <order> can be either "asc" for ascending
                                     and "desc" for descending.
    -i, --itemlist                   Output identifiers only.
    -f, --field=<field>...           Metadata fields to return.
    -n, --number-found               Print the number of results to stdout.
    --v2                             Search https://archive.org/v2.

"""
import sys
try:
    import ujson as json
except ImportError:
    import json

from docopt import docopt

from internetarchive import search_items


# main()
#_________________________________________________________________________________________
def main(argv):
    args = docopt(__doc__, argv=argv)

    params = dict(p.split(':') for p in args['--parameters'])

    # format sort paramaters.
    if args['--sort']:
        for i, field in enumerate(args['--sort']):
            key = 'sort[{0}]'.format(i)
            params[key] = field.strip().replace(':', ' ')

    query = ' '.join(args['<query>'])
    if args['--itemlist']:
        fields = ['identifier']
    else:
        fields = args['--field']
    search = search_items(query, fields=args['--field'], params=params, v2=args['--v2'])
    if args['--number-found']:
        sys.stdout.write('{0}\n'.format(search.num_found))
        sys.exit(0)
    for result in search:
        try:
            if args['--itemlist']:
                sys.stdout.write(result.get('identifier', ''))
            else:
                json.dump(result, sys.stdout)
            sys.stdout.write('\n')
        except IOError:
            sys.exit(0)



"""Configure the `ia` CLI and internetarchive Python library.

usage:
    ia configure [--help]

options:
    -h, --help

"""
import os
import sys
from getpass import getpass

from docopt import docopt
import yaml
from requests.exceptions import HTTPError

from internetarchive.config import get_auth_config



# ia_configure()
# ________________________________________________________________________________________
def main(argv):
    args = docopt(__doc__, argv=argv)

    sys.stdout.write('Please enter your Archive.org credentials below to have your\n'
                     'Archive.org cookies and IA-S3 keys added to your config file.\n\n')
    username=raw_input('Email address: '),
    password=getpass('Password: '),

    try:
        config = get_auth_config(username, password)
    except HTTPError as exc:
        sys.stderr.write('\n{0}\n'.format(str(exc)))
        sys.exit(1)

    configfile = yaml.dump(config, default_flow_style=False)
    configdir = os.path.join(os.environ['HOME'], '.config')
    if not os.path.isdir(configdir) and not os.path.isfile(configdir):
        os.mkdir(configdir)

    filename = ''
    if os.path.isdir(configdir):
        filename = os.path.join(configdir, 'internetarchive.yml')
    else:
        filename = os.path.join(os.environ['HOME'], '.internetarchive.yml')

    if os.path.exists(filename):
        overwrite = raw_input('\nYou already have an ia config file: '
                              '{0} \n\nWould you like to overwrite it?'
                              '[y/n] '.format(filename).lower())
        if overwrite not in ['y', 'yes']:
            sys.stdout.write('\nExiting without overwriting config file!\n')
            sys.exit(1)

    with open(filename, 'wb') as fp:
        os.chmod(filename, 0o700)
        fp.write(configfile)

    sys.stdout.write('\nSuccessfully saved your new config to: {0}\n'.format(filename))



import os, sys
from subprocess import Popen, PIPE
from time import time

import pytest

inc_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0, inc_path)
import internetarchive.config


@pytest.mark.skipif('internetarchive.config.get_config().get("s3") is None',
                    reason='requires authorization.')
def test_ia_upload():
    # upload from stdin.
    cmd = ('echo "Hello World!" |'
           'ia upload iacli-test-item - --remote-name="stdin.txt" --size-hint=8')
    proc = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE)
    stdout, stderr = proc.communicate()
    assert proc.returncode == 0

    cmd = ('echo "Hello World!" |'
           'ia upload iacli-test-item -')
    proc = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE)
    stdout, stderr = proc.communicate()
    assert proc.returncode == 1
    assert stderr == '--remote-name is required when uploading from stdin.\n'

    # upload file.
    cmd = 'ia upload iacli-test-item setup.py'
    proc = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE)
    stdout, stderr = proc.communicate()
    assert proc.returncode == 0

    # upload debug.
    cmd = 'ia upload iacli-test-item setup.py --debug'
    proc = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE)
    stdout, stderr = proc.communicate()
    assert proc.returncode == 0

    # upload non-200 status_code.
    cmd = ('echo "Hello World!" |'
           'ia upload iacli-test-item - --remote-name="iacli-test-item_meta.xml"')
    proc = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE)
    stdout, stderr = proc.communicate()
    assert stderr == 'error "AccessDenied" (403): Access Denied\n'
    assert proc.returncode == 1



# -*- coding: utf-8 -*-

import os, sys, shutil, string
inc_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0, inc_path)

import internetarchive.utils

def test_utils():
    cg = list(internetarchive.utils.chunk_generator(open('setup.py'), 10))
    ifp = internetarchive.utils.IterableToFileAdapter([1, 2], 200)
    assert len(ifp) == 200
    ifp.read()


def test_needs_quote():
    notascii = 'ȧƈƈḗƞŧḗḓ ŧḗẋŧ ƒǿř ŧḗşŧīƞɠ, ℛℯα∂α♭ℓℯ ♭ʊ☂ η☺т Ѧ$☾ℐℐ, ¡ooʇ ןnɟǝsn sı uʍop-ǝpısdn'
    assert internetarchive.utils.needs_quote(notascii)
    assert internetarchive.utils.needs_quote(string.whitespace)
    assert not internetarchive.utils.needs_quote(string.ascii_letters + string.digits)




import requests.sessions

from . import session


# Search class
# ________________________________________________________________________________________
class Search(object):
    """This class represents an archive.org item search. You can use
    this class to search for archive.org items using the advanced
    search engine.

    Usage::

        >>> import internetarchive.search
        >>> search = internetarchive.search.Search('(uploader:jake@archive.org)')
        >>> for result in search:
        ...     print(result['identifier'])

    """
    # init()
    # ____________________________________________________________________________________
    def __init__(self, query, fields=['identifier'], params={}, config=None, v2=False):
        self.session = session.ArchiveSession(config)
        self.http_session = requests.sessions.Session()
        self.url = 'http://archive.org/advancedsearch.php'
        default_params = dict(
            q=query,
            rows=100,
        )

        if v2:
            # Use "1" as value to not confuse IA analytics.
            self.session.cookies['ui3'] = '1'
            self.http_session.cookies = self.session.cookies

        self.params = default_params.copy()
        self.params.update(params)
        if not self.params.get('output'):
            self.params['output'] = 'json'

        for k, v in enumerate(fields):
            key = 'fl[{0}]'.format(k)
            self.params[key] = v
        self._search_info = self._get_search_info()
        self.num_found = self._search_info['response']['numFound']
        self.query = self._search_info['responseHeader']['params']['q']

    # __repr__()
    # ____________________________________________________________________________________
    def __repr__(self):
        return ('Search(query={query!r}, '
                'num_found={num_found!r})'.format(**self.__dict__))

    # _get_search_info()
    # ____________________________________________________________________________________
    def _get_search_info(self):
        info_params = self.params.copy()
        info_params['rows'] = 0
        r = self.http_session.get(self.url, params=self.params)
        results = r.json()
        del results['response']['docs']
        return results

    # __iter__()
    # ____________________________________________________________________________________
    def __iter__(self):
        """Generator for iterating over search results"""
        total_pages = ((self.num_found // self.params['rows']) + 2)
        for page in range(1, total_pages):
            self.params['page'] = page
            r = self.http_session.get(self.url, params=self.params)
            results = r.json()
            for doc in results['response']['docs']:
                yield doc



import os, sys
from subprocess import Popen, PIPE
from time import time

import pytest

inc_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0, inc_path)
import internetarchive.config


def test_ia_metadata_exists():
    cmd = 'ia metadata --exists iacli_test-doesnotexist'
    proc = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE)
    stdout, stderr = proc.communicate()
    assert proc.returncode == 1

    cmd = 'ia metadata --exists nasa'
    proc = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE)
    stdout, stderr = proc.communicate()
    assert proc.returncode == 0

def test_ia_metadata_formats():
    cmd = 'ia metadata --formats iacli-test-item60'
    proc = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE)
    stdout, stderr = proc.communicate()
    test_output_set = set([
        "Text",
        "Metadata",
        "Unknown",
    ])
    assert set(stdout[:-1].split('\n')) == test_output_set

@pytest.mark.skipif('internetarchive.config.get_config().get("cookies") == None',
                    reason='requires authorization.')
def test_ia_metadata_modify():
    # Modify test item.
    valid_key = "foo-{k}".format(k=int(time())) 
    cmd = 'ia metadata --modify="{k}:test_value" iacli-test-item60'.format(k=valid_key)
    proc = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE)
    stdout, stderr = proc.communicate()
    assert proc.returncode == 0

    # Submit illegal modification.
    cmd = 'ia metadata --modify="-foo:test_value" iacli-test-item60'
    proc = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE)
    stdout, stderr = proc.communicate()
    assert proc.returncode == 1
    assert "Illegal tag name" in stderr



"""Retrieve information about your catalog tasks.

usage:
    ia tasks [--verbose] [--task=<task_id>...] [--get-task-log=<task_id>]
             [--green-rows] [--blue-rows] [--red-rows] [--parameter=<k:v>...]
    ia tasks [--verbose] <identifier>
             [--green-rows] [--blue-rows] [--red-rows] [--parameter=<k:v>...]
    ia tasks --help

options:
    -h, --help
    -v, --verbose                 Ouptut detailed information for each task.
    -t, --task=<task_id>...       Return information about the given task.
    -G, --get-task-log=<task_id>  Return the given tasks task log.
    -g, --green-rows              Return information about tasks that have not run.
    -b, --blue-rows               Return information about running tasks.
    -r, --red-rows                Return information about tasks that have failed.
    -p, --parameter=<k:v>...      Return tasks matching the given parameter.

"""
import sys

from docopt import docopt

from internetarchive import Catalog, get_tasks
from internetarchive.iacli.argparser import get_args_dict


# ia_catalog()
#_________________________________________________________________________________________
def main(argv):
    args = docopt(__doc__, argv=argv)
    params = get_args_dict(args['--parameter'])

    row_types = {
        -1: 'done',
        0: 'green',
        1: 'blue',
        2: 'red',
        9: 'brown',
    }

    task_type = None
    if args['--green-rows']:
        task_type = 'green'
    elif args['--blue-rows']:
        task_type = 'blue'
    elif args['--red-rows']:
        task_type = 'red'

    try:
        if args['<identifier>']:
            tasks = get_tasks(identifier=args['<identifier>'], task_type=task_type, params=params)
        elif args['--get-task-log']:
            task = get_tasks(task_ids=args['--get-task-log'], params=params)
            if task:
                log = task[0].task_log()
                sys.stdout.write(log)
            else:
                sys.stderr.write(
                    'error retrieving task-log for {0}\n'.format(args['--get-task-log'])
                )
                sys.exit(1)
            sys.exit(0)
        elif args['--task']:
            tasks = get_tasks(task_ids=args['--task'], params=params)
        else:
            tasks = get_tasks(task_type=task_type, params=params)

        for t in tasks:
            task_info = [
                t.identifier, t.task_id, t.server, t.time, t.command, 
                row_types[t.row_type],
            ]
            if args['--verbose']:
                # parse task args and append to task_info list.
                targs = '\t'.join(['{0}={1}'.format(k, v) for (k, v) in t.args.items()])
                task_info += [t.submitter, targs]
            sys.stdout.write('\t'.join([str(x) for x in task_info]) + '\n')
    except NameError as exc:
        sys.stderr.write('error: {0}'.format(exc.message))
        sys.exit(1)



import hashlib
import os
import re


def needs_quote(s):
    try:
        s.encode('ascii')
    except (UnicodeDecodeError, UnicodeEncodeError):
        return True
    return re.search(r'\s', s) is not None


def get_md5(file_object):
    m = hashlib.md5()
    while True:
        data = file_object.read(8192)
        if not data:
            break
        m.update(data)
    file_object.seek(0, os.SEEK_SET)
    return m.hexdigest()


def chunk_generator(fp, chunk_size):
    while True:
        chunk = fp.read(chunk_size)
        if not chunk:
            break
        yield chunk


class IterableToFileAdapter(object):
    def __init__(self, iterable, size):
        self.iterator = iter(iterable)
        self.length = size

    def read(self, size=-1):  # TBD: add buffer for `len(data) > size` case
        return next(self.iterator, b'')

    def __len__(self):
        return self.length



import os, sys, shutil
from time import time

import pytest

inc_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0, inc_path)
from internetarchive.api import *
import internetarchive.config

try:
    import internetarchive.mine
    py_test_mine = True
except ImportError:
    py_test_mine = False

def test_api():
    # get_item()
    item = get_item('iacli-test-item')
    assert item.metadata['identifier'] == 'iacli-test-item'

    # get_files()
    files = get_files('nasa', 'nasa_meta.xml')
    assert files[0].name == 'nasa_meta.xml'

    md_files = ['nasa_meta.xml', 'nasa_files.xml']
    files = get_files('nasa', md_files)
    assert all(f.name in md_files for f in files)
    
    og_files = ['NASAarchiveLogo.jpg', 'globe_west_540.jpg']
    files = get_files('nasa', source='original')
    assert all(f.name in og_files for f in files)

    og_files = ['NASAarchiveLogo.jpg', 'globe_west_540.jpg']
    files = get_files('nasa', formats='Archive BitTorrent')
    assert files[0].name == 'nasa_archive.torrent'

    xml_files = ['nasa_meta.xml', 'nasa_reviews.xml', 'nasa_files.xml']
    files = get_files('nasa', glob_pattern='*xml')
    assert all(f.name in xml_files for f in files)

    # iter_files()
    file_generator = iter_files('nasa')
    assert not isinstance(file_generator, list)
    all_files = ['NASAarchiveLogo.jpg', 'globe_west_540.jpg', 'nasa_reviews.xml',
                 'nasa_meta.xml', 'nasa_archive.torrent', 'nasa_files.xml',
                 'globe_west_540_thumb.jpg']
    assert all(f.name in all_files for f in file_generator)

    # download()
    if os.path.exists('iacli-test-item'):
        shutil.rmtree('iacli-test-item')
    r = download('iacli-test-item')
    assert os.path.exists('iacli-test-item')
    shutil.rmtree('iacli-test-item')

    # download_file()
    r = download('nasa', 'nasa_meta.xml')
    os.remove('nasa_meta.xml')

    # search()
    s = search_items('identifier:nasa')
    assert s.num_found == 1

@pytest.mark.skipif('internetarchive.config.get_config().get("cookies") == None',
                    reason='requires authorization.')
def test_functions_needing_cookies():
    # modify_metadata()
    valid_key = "foo-{k}".format(k=int(time()))
    md = {
        valid_key: 'test value'
    }
    r = modify_metadata('iacli-test-item', md)

    # Remove tag.
    md = {
        valid_key: 'REMOVE_TAG'
    }
    r = modify_metadata('iacli-test-item', md)

    tasks = get_tasks()
    red_rows = get_tasks(task_type='red')

@pytest.mark.skipif('internetarchive.config.get_config().get("s3") == None',
                    reason='requires authorization.')
def test_funtions_needing_s3_keys():
    r = upload('iacli-test-item', {'testsetup.py': 'setup.py'})



@pytest.mark.skipif('py_test_mine == False', reason='requires internetarchive[speedups]')
def test_mine():
    ids = ['iacli-test-item', 'nasa']
    miner = get_data_miner(ids)
    for i, item in miner:
        assert item.exists



import os

import requests
import yaml



# get_config()
# ____________________________________________________________________________________
def get_auth_config(username, password):
    payload = dict(
        username=username,
        password=password,
        remember='CHECKED',
        action='login',
    )

    with requests.Session() as s:
        # Attache logged-in-* cookies to Session.
        u = 'https://archive.org/account/login.php'
        r = s.post(u, data=payload, cookies={'test-cookie': '1'})

        # Get S3 keys.
        u = 'https://archive.org/account/s3.php'
        p = dict(output_json=1)
        r = s.get(u, params=p)
        j = r.json()

        if not j or not j.get('key'):
            raise requests.exceptions.HTTPError(
                'Authorization failed. Please check your credentials and try again.')

        auth_config = {
            's3': {
                'access_key': j['key']['s3accesskey'],
                'secret_key': j['key']['s3secretkey'],
            },
            'cookies': {
                'logged-in-user': s.cookies['logged-in-user'],
                'logged-in-sig': s.cookies['logged-in-sig'],
            }
        }

    return auth_config


# get_config()
# ____________________________________________________________________________________
def get_config(config=None, config_file=None):
    config = {} if not config else config
    if not config_file:
        home_dir = os.environ.get('HOME')
        if not home_dir:
            return config
        config_file = os.path.join(home_dir, '.config', 'internetarchive.yml')
    try:
        _config = yaml.load(open(config_file))
    except IOError:
        config_file = os.path.join(home_dir, '.internetarchive.yml')
        try:
            _config = yaml.load(open(config_file))
        except IOError:
            _config = {}
    final_config = _config.copy()
    final_config.update(config)
    return final_config



# -*- coding: utf-8 -*-
"""
Internetarchive Library
~~~~~~~~~~~~~~~~~~~~~~~

Internetarchive is a python interface to archive.org.
usage:

    >>> from internetarchive import get_item
    >>> item = get_item('govlawgacode20071')
    >>> item.exists
    True

:copyright: (c) 2015 by Internet Archive.
:license: AGPL 3, see LICENSE for more details.

"""

__title__ = 'internetarchive'
__version__ = '0.9.2'
__author__ = 'Jacob M. Johnson'
__license__ = 'AGPL 3'
__copyright__ = 'Copyright 2015 Internet Archive'

from .item import Item, File
from .search import Search
from .catalog import Catalog
from .api import *


# Set default logging handler to avoid "No handler found" warnings.
import logging
try:  # Python 2.7+
    from logging import NullHandler
except ImportError:
    class NullHandler(logging.Handler):
        def emit(self, record):
            pass


log = logging.getLogger('internetarchive')
log.addHandler(NullHandler())



import os
import sys
from fnmatch import fnmatch
import logging
import time

import requests.sessions
from requests.adapters import HTTPAdapter
from requests.exceptions import HTTPError
from requests import Response
from clint.textui import progress
import six
import six.moves.urllib as urllib

from . import __version__, session, iarequest, utils


log = logging.getLogger(__name__)


# Item class
# ________________________________________________________________________________________
class Item(object):
    """This class represents an archive.org item. You can use this
    class to access item metadata::

        >>> import internetarchive
        >>> item = internetarchive.Item('stairs')
        >>> print(item.metadata)

    Or to modify the metadata for an item::

        >>> metadata = dict(title='The Stairs')
        >>> item.modify(metadata)
        >>> print(item.metadata['title'])
        u'The Stairs'

    This class also uses IA's S3-like interface to upload files to an
    item. You need to supply your IAS3 credentials in environment
    variables in order to upload::

        >>> item.upload('myfile.tar', access_key='Y6oUrAcCEs4sK8ey',
        ...                           secret_key='youRSECRETKEYzZzZ')
        True

    You can retrieve S3 keys here: `https://archive.org/account/s3.php
    <https://archive.org/account/s3.php>`__

    """
    # init()
    # ____________________________________________________________________________________
    def __init__(self, identifier, metadata_timeout=None, config=None, max_retries=1,
                 archive_session=None):
        """
        :type identifier: str
        :param identifier: The globally unique Archive.org identifier
                           for a given item.

        :type metadata_timeout: int
        :param metadata_timeout: (optional) Set a timeout for retrieving
                                 an item's metadata.

        :type config: dict
        :param secure: (optional) Configuration options for session.

        :type max_retries: int
        :param max_retries: (optional) Maximum number of times to request
                            a website if the connection drops. (default: 1)

        :type archive_session: :class:`ArchiveSession <ArchiveSession>`
        :param archive_session: An :class:`ArchiveSession <ArchiveSession>`
                                object can be provided via the `archive_session`
                                parameter.

        """
        self.session = archive_session if archive_session else session.get_session(config)
        self.protocol = 'https:' if self.session.secure else 'http:'
        self.http_session = requests.sessions.Session()
        max_retries_adapter = HTTPAdapter(max_retries=max_retries)
        self.http_session.mount('{0}//'.format(self.protocol), max_retries_adapter)
        self.http_session.cookies = self.session.cookies
        self.identifier = identifier

        # Default empty attributes.
        self.metadata = {}
        self.files = []
        self.created = None
        self.d1 = None
        self.d2 = None
        self.dir = None
        self.files_count = None
        self.item_size = None
        self.reviews = []
        self.server = None
        self.uniq = None
        self.updated = None
        self.tasks = None

        self._json = self.get_metadata(metadata_timeout)
        self.exists = False if self._json == {} else True

    # __repr__()
    # ____________________________________________________________________________________
    def __repr__(self):
        return ('Item(identifier={identifier!r}, '
                'exists={exists!r})'.format(**self.__dict__))

    # get_metadata()
    # ____________________________________________________________________________________
    def get_metadata(self, metadata_timeout=None):
        """Get an item's metadata from the `Metadata API
        <http://blog.archive.org/2013/07/04/metadata-api/>`__

        :type identifier: str
        :param identifier: Globally unique Archive.org identifier.

        :rtype: dict
        :returns: Metadat API response.

        """
        url = '{protocol}//archive.org/metadata/{identifier}'.format(**self.__dict__)
        try:
            resp = self.http_session.get(url, timeout=metadata_timeout)
            resp.raise_for_status()
        except HTTPError as e:
            error_msg = 'Error retrieving metadata from {0}, {1}'.format(resp.url, e)
            log.error(error_msg)
            raise HTTPError(error_msg)
        metadata = resp.json()
        for key in metadata:
                setattr(self, key, metadata[key])
        return metadata

    # iter_files()
    # ____________________________________________________________________________________
    def iter_files(self):
        """Generator for iterating over files in an item.

        :rtype: generator
        :returns: A generator that yields :class:`internetarchive.File
                  <File>` objects.

        """
        for file_dict in self.files:
            file = File(self, file_dict.get('name'))
            yield file

    # file()
    # ____________________________________________________________________________________
    def get_file(self, file_name):
        """Get a :class:`File <File>` object for the named file.

        :rtype: :class:`internetarchive.File <File>`
        :returns: An :class:`internetarchive.File <File>` object.

        """
        for f in self.iter_files():
            if f.name == file_name:
                return f

    # get_files()
    # ____________________________________________________________________________________
    def get_files(self, files=None, source=None, formats=None, glob_pattern=None):
        files = [] if not files else files
        source = [] if not source else source

        if not isinstance(files, (list, tuple, set)):
            files = [files]
        if not isinstance(source, (list, tuple, set)):
            source = [source]
        if not isinstance(formats, (list, tuple, set)):
            formats = [formats]

        file_objects = []
        for f in self.iter_files():
            if f.name in files:
                file_objects.append(f)
            elif f.source in source:
                file_objects.append(f)
            elif f.format in formats:
                file_objects.append(f)
            elif glob_pattern:
                # Support for | operator.
                patterns = glob_pattern.split('|')
                if not isinstance(patterns, list):
                    patterns = [patterns]
                for p in patterns:
                    if fnmatch(f.name, p):
                        file_objects.append(f)
        return file_objects

    # download()
    # ____________________________________________________________________________________
    def download(self, concurrent=None, source=None, formats=None, glob_pattern=None,
                 dry_run=None, verbose=None, ignore_existing=None, checksum=None,
                 destdir=None, no_directory=None):
        """Download the entire item into the current working directory.

        :type concurrent: bool
        :param concurrent: Download files concurrently if ``True``.

        :type source: str
        :param source: Only download files matching given source.

        :type formats: str
        :param formats: Only download files matching the given Formats.

        :type glob_pattern: str
        :param glob_pattern: Only download files matching the given glob
                             pattern

        :type ignore_existing: bool
        :param ignore_existing: Overwrite local files if they already
                                exist.

        :type checksum: bool
        :param checksum: Skip downloading file based on checksum.

        :type no_directory: bool
        :param no_directory: Download files to current working
                             directory rather than creating an item
                             directory.

        :rtype: bool
        :returns: True if if files have been downloaded successfully.

        """
        concurrent = False if concurrent is None else concurrent
        dry_run = False if dry_run is None else dry_run
        verbose = False if verbose is None else verbose
        ignore_existing = False if ignore_existing is None else ignore_existing
        checksum = False if checksum is None else checksum
        no_directory = False if no_directory is None else no_directory

        if verbose:
            sys.stdout.write('{0}:\n'.format(self.identifier))
            if self._json.get('is_dark') is True:
                sys.stdout.write(' skipping: item is dark.\n')
                log.warning('Not downloading item {0}, '
                            'item is dark'.format(self.identifier))
            elif self.metadata == {}:
                sys.stdout.write(' skipping: item does not exist.\n')
                log.warning('Not downloading item {0}, '
                            'item does not exist.'.format(self.identifier))

        if concurrent:
            try:
                from gevent import monkey
                monkey.patch_socket()
                from gevent.pool import Pool
                pool = Pool()
            except ImportError:
                raise ImportError(
                    """No module named gevent

                    Downloading files concurrently requires the gevent neworking library.
                    gevent and all of it's dependencies can be installed with pip:

                    \tpip install cython git+git://github.com/surfly/gevent.git@1.0rc2#egg=gevent

                    """)

        files = self.iter_files()
        if source:
            files = self.get_files(source=source)
        if formats:
            files = self.get_files(formats=formats)
        if glob_pattern:
            files = self.get_files(glob_pattern=glob_pattern)

        if not files and verbose:
            sys.stdout.write(' no matching files found, nothing downloaded.\n')
        for f in files:
            fname = f.name.encode('utf-8')
            if no_directory:
                path = fname
            else:
                path = os.path.join(self.identifier, fname)
            if dry_run:
                sys.stdout.write(f.url + '\n')
                continue
            if concurrent:
                pool.spawn(f.download, path, verbose, ignore_existing, checksum, destdir)
            else:
                f.download(path, verbose, ignore_existing, checksum, destdir)
        if concurrent:
            pool.join()
        return True

    # modify_metadata()
    # ____________________________________________________________________________________
    def modify_metadata(self, metadata, target=None, append=False, priority=None,
                        access_key=None, secret_key=None, debug=False):
        """Modify the metadata of an existing item on Archive.org.

        Note: The Metadata Write API does not yet comply with the
        latest Json-Patch standard. It currently complies with `version 02
        <https://tools.ietf.org/html/draft-ietf-appsawg-json-patch-02>`__.

        :type metadata: dict
        :param metadata: Metadata used to update the item.

        :type target: str
        :param target: (optional) Set the metadata target to update.

        :type priority: int
        :param priority: (optional) Set task priority.

        Usage::

            >>> import internetarchive
            >>> item = internetarchive.Item('mapi_test_item1')
            >>> md = dict(new_key='new_value', foo=['bar', 'bar2'])
            >>> item.modify_metadata(md)

        :rtype: dict
        :returns: A dictionary containing the status_code and response
                  returned from the Metadata API.

        """
        access_key = self.session.access_key if not access_key else access_key
        secret_key = self.session.secret_key if not secret_key else secret_key
        target = 'metadata' if target is None else target

        url = '{protocol}//archive.org/metadata/{identifier}'.format(**self.__dict__)
        request = iarequest.MetadataRequest(
            url=url,
            metadata=metadata,
            source_metadata=self._json.get(target.split('/')[0], {}),
            target=target,
            priority=priority,
            access_key=access_key,
            secret_key=secret_key,
            append=append,
        )
        if debug:
            return request
        prepared_request = request.prepare()
        resp = self.http_session.send(prepared_request)
        self._json = self.get_metadata()
        return resp

    # s3_is_overloaded()
    # ____________________________________________________________________________________
    def s3_is_overloaded(self, access_key=None):
        u = 'http://s3.us.archive.org'
        p = dict(
            check_limit=1,
            accesskey=access_key,
            bucket=self.identifier,
        )
        r = self.http_session.get(u, params=p)
        j = r.json()
        if j.get('over_limit') == 0:
            return False
        else:
            return True

    # upload_file()
    # ____________________________________________________________________________________
    def upload_file(self, body, key=None, metadata=None, headers=None,
                    access_key=None, secret_key=None, queue_derive=True,
                    ignore_preexisting_bucket=False, verbose=False, verify=True,
                    checksum=False, delete=False, retries=None, retries_sleep=None,
                    debug=False, **kwargs):
        """Upload a single file to an item. The item will be created
        if it does not exist.

        :type body: Filepath or file-like object.
        :param body: File or data to be uploaded.

        :type key: str
        :param key: (optional) Remote filename.

        :type metadata: dict
        :param metadata: (optional) Metadata used to create a new item.

        :type headers: dict
        :param headers: (optional) Add additional IA-S3 headers to request.

        :type queue_derive: bool
        :param queue_derive: (optional) Set to False to prevent an item from
                             being derived after upload.

        :type ignore_preexisting_bucket: bool
        :param ignore_preexisting_bucket: (optional) Destroy and respecify the
                                          metadata for an item

        :type verify: bool
        :param verify: (optional) Verify local MD5 checksum matches the MD5
                       checksum of the file received by IAS3.

        :type checksum: bool
        :param checksum: (optional) Skip based on checksum.

        :type delete: bool
        :param delete: (optional) Delete local file after the upload has been
                       successfully verified.

        :type retries: int
        :param retries: (optional) Number of times to retry the given request
                        if S3 returns a 503 SlowDown error.

        :type retries_sleep: int
        :param retries_sleep: (optional) Amount of time to sleep between
                              ``retries``.

        :type verbose: bool
        :param verbose: (optional) Print progress to stdout.

        :type debug: bool
        :param debug: (optional) Set to True to print headers to stdout, and
                      exit without sending the upload request.

        Usage::

            >>> import internetarchive
            >>> item = internetarchive.Item('identifier')
            >>> item.upload_file('/path/to/image.jpg',
            ...                  key='photos/image1.jpg')
            True

        """
        # Defaults for empty params.
        headers = {} if headers is None else headers
        metadata = {} if metadata is None else metadata
        access_key = self.session.access_key if access_key is None else access_key
        secret_key = self.session.secret_key if secret_key is None else secret_key
        retries = 0 if retries is None else retries
        retries_sleep = 30 if retries_sleep is None else retries_sleep

        if not hasattr(body, 'read'):
            body = open(body, 'rb')

        if not metadata.get('scanner'):
            scanner = 'Internet Archive Python library {0}'.format(__version__)
            metadata['scanner'] = scanner

        try:
            body.seek(0, os.SEEK_END)
            size = body.tell()
            body.seek(0, os.SEEK_SET)
        except IOError:
            size = None

        if not headers.get('x-archive-size-hint'):
            headers['x-archive-size-hint'] = size

        key = body.name.split('/')[-1] if key is None else key
        base_url = '{protocol}//s3.us.archive.org/{identifier}'.format(**self.__dict__)
        url = '{base_url}/{key}'.format(base_url=base_url, key=urllib.parse.quote(key))

        # Skip based on checksum.
        md5_sum = utils.get_md5(body)
        ia_file = self.get_file(key)
        if (checksum) and (not self.tasks) and (ia_file) and (ia_file.md5 == md5_sum):
            log.info('{f} already exists: {u}'.format(f=key, u=url))
            if verbose:
                sys.stdout.write(' {f} already exists, skipping.\n'.format(f=key))
            if delete:
                log.info(
                    '{f} successfully uploaded to https://archive.org/download/{i}/{f} '
                    'and verified, deleting '
                    'local copy'.format(i=self.identifier, f=key)
                )
                os.remove(body.name)
            # Return an empty response object if checksums match.
            # TODO: Is there a better way to handle this?
            return Response()

        # require the Content-MD5 header when delete is True.
        if verify or delete:
            headers['Content-MD5'] = md5_sum

        # Delete retries and sleep_retries from kwargs.
        if 'retries' in kwargs:
            del kwargs['retries']
        if 'retries_sleep' in kwargs:
            del kwargs['retries_sleep']

        def _build_request():
            body.seek(0, os.SEEK_SET)
            if verbose:
                try:
                    chunk_size = 1048576
                    expected_size = size/chunk_size + 1
                    chunks = utils.chunk_generator(body, chunk_size)
                    progress_generator = progress.bar(chunks, expected_size=expected_size,
                                                      label=' uploading {f}: '.format(f=key))
                    data = utils.IterableToFileAdapter(progress_generator, size)
                except:
                    sys.stdout.write(' uploading {f}: '.format(f=key))
                    data = body
            else:
                data = body

            request = iarequest.S3Request(
                method='PUT',
                url=url,
                headers=headers,
                data=data,
                metadata=metadata,
                access_key=access_key,
                secret_key=secret_key,
                queue_derive=queue_derive,
                **kwargs
            )
            return request

        if debug:
            return _build_request()
        else:
            try:
                error_msg = ('s3 is overloaded, sleeping for '
                             '{0} seconds and retrying. '
                             '{1} retries left.'.format(retries_sleep, retries))
                while True:
                    if retries > 0:
                        if self.s3_is_overloaded(access_key):
                            time.sleep(retries_sleep)
                            log.info(error_msg)
                            if verbose:
                                sys.stderr.write(' warning: {0}\n'.format(error_msg))
                            retries -= 1
                            continue
                    request = _build_request()
                    prepared_request = request.prepare()
                    response = self.http_session.send(prepared_request, stream=True)
                    if (response.status_code == 503) and (retries > 0):
                        log.info(error_msg)
                        if verbose:
                            sys.stderr.write(' warning: {0}\n'.format(error_msg))
                        time.sleep(retries_sleep)
                        retries -= 1
                        continue
                    else:
                        if response.status_code == 503:
                            log.info('maximum retries exceeded, upload failed.')
                        break
                response.raise_for_status()
                log.info('uploaded {f} to {u}'.format(f=key, u=url))
                if delete and response.status_code == 200:
                    log.info(
                        '{f} successfully uploaded to '
                        'https://archive.org/download/{i}/{f} and verified, deleting '
                        'local copy'.format(i=self.identifier, f=key)
                    )
                    os.remove(body.name)
                return response
            except HTTPError as exc:
                error_msg = (' error uploading {0} to {1}, '
                             '{2}'.format(key, self.identifier, exc))
                log.error(error_msg)
                if verbose:
                    sys.stderr.write(error_msg + '\n')
                # Raise HTTPError with error message.
                raise type(exc)(error_msg)

    # upload()
    # ____________________________________________________________________________________
    def upload(self, files, **kwargs):
        """Upload files to an item. The item will be created if it
        does not exist.

        :type files: list
        :param files: The filepaths or file-like objects to upload.

        :type kwargs: dict
        :param kwargs: The keyword arguments from the call to
                       upload_file().

        Usage::

            >>> import internetarchive
            >>> item = internetarchive.Item('identifier')
            >>> md = dict(mediatype='image', creator='Jake Johnson')
            >>> item.upload('/path/to/image.jpg', metadata=md, queue_derive=False)
            True

        :rtype: bool
        :returns: True if the request was successful and all files were
                  uploaded, False otherwise.

        """
        def iter_directory(directory):
            for path, dir, files in os.walk(directory):
                for f in files:
                    filepath = os.path.join(path, f)
                    key = os.path.relpath(filepath, directory)
                    yield (filepath, key)

        if isinstance(files, dict):
            files = files.items()
        if not isinstance(files, (list, tuple)):
            files = [files]

        queue_derive = kwargs.get('queue_derive', True)

        responses = []
        file_index = 0
        for f in files:
            file_index += 1
            if isinstance(f, six.string_types) and os.path.isdir(f):
                fdir_index = 0
                for filepath, key in iter_directory(f):
                    # Set derive header if queue_derive is True,
                    # and this is the last request being made.
                    fdir_index += 1
                    if queue_derive is True and file_index >= len(files) \
                        and fdir_index >= len(os.listdir(f)):
                            kwargs['queue_derive'] = True
                    else:
                        kwargs['queue_derive'] = False

                    if not f.endswith('/'):
                        key = '{0}/{1}'.format(f, key)
                    resp = self.upload_file(filepath, key=key, **kwargs)
                    responses.append(resp)
            else:
                # Set derive header if queue_derive is True,
                # and this is the last request being made.
                if queue_derive is True and file_index >= len(files):
                    kwargs['queue_derive'] = True
                else:
                    kwargs['queue_derive'] = False

                if not isinstance(f, (list, tuple)):
                    key, body = (None, f)
                else:
                    key, body = f
                if key and not isinstance(key, six.string_types):
                    raise ValueError('Key must be a string.')
                resp = self.upload_file(body, key=key, **kwargs)
                responses.append(resp)
        return responses


# File class
# ________________________________________________________________________________________
class File(object):
    """This class represents a file in an archive.org item. You
    can use this class to access the file metadata::

        >>> import internetarchive
        >>> item = internetarchive.Item('stairs')
        >>> file = internetarchive.File(item, 'stairs.avi')
        >>> print(f.format, f.size)
        (u'Cinepack', u'3786730')

    Or to download a file::

        >>> file.download()
        >>> file.download('fabulous_movie_of_stairs.avi')

    This class also uses IA's S3-like interface to delete a file
    from an item. You need to supply your IAS3 credentials in
    environment variables in order to delete::

        >>> file.delete(access_key='Y6oUrAcCEs4sK8ey',
        ...             secret_key='youRSECRETKEYzZzZ')

    You can retrieve S3 keys here: `https://archive.org/account/s3.php
    <https://archive.org/account/s3.php>`__

    """
    # init()
    # ____________________________________________________________________________________
    def __init__(self, item, name):
        """
        :type item: Item
        :param item: The item that the file is part of.

        :type name: str
        :param name: The filename of the file.

        """
        _file = {}
        for f in item.files:
            if f.get('name') == name:
                _file = f
                break

        self._item = item
        self.identifier = item.identifier
        self.name = None
        self.size = None
        self.source = None
        self.format = None
        self.md5 = None
        for key in _file:
            setattr(self, key, _file[key])
        base_url = '{protocol}//archive.org/download/{identifier}'.format(**item.__dict__)
        self.url = '{base_url}/{name}'.format(base_url=base_url,
                                              name=urllib.parse.quote(name.encode('utf-8')))

    # __repr__()
    # ____________________________________________________________________________________
    def __repr__(self):
        return ('File(identifier={identifier!r}, '
                'filename={name!r}, '
                'size={size!r}, '
                'source={source!r}, '
                'format={format!r})'.format(**self.__dict__))

    # download()
    # ____________________________________________________________________________________
    def download(self, file_path=None, verbose=None, ignore_existing=None, checksum=None,
                 destdir=None):
        """Download the file into the current working directory.

        :type file_path: str
        :param file_path: Download file to the given file_path.

        :type ignore_existing: bool
        :param ignore_existing: Overwrite local files if they already
                                exist.

        :type checksum: bool
        :param checksum: Skip downloading file based on checksum.

        """
        verbose = False if verbose is None else verbose
        ignore_existing = False if ignore_existing is None else ignore_existing
        checksum = False if checksum is None else checksum

        file_path = self.name if not file_path else file_path

        if destdir:
            if not os.path.exists(destdir):
                os.mkdir(destdir)
            if os.path.isfile(destdir):
                raise IOError('{} is not a directory!'.format(destdir))
            file_path = os.path.join(destdir, file_path)

        if os.path.exists(file_path):
            if ignore_existing is False and checksum is False:
                raise IOError('file already downloaded: {0}'.format(file_path))
            if checksum:
                md5_sum = utils.get_md5(open(file_path))
                if md5_sum == self.md5:
                    log.info('not downloading file {0}, '
                             'file already exists.'.format(file_path))
                    if verbose:
                        sys.stdout.write(' skipping {0}: already exists.\n'.format(file_path))
                    return

        if verbose:
            sys.stdout.write(' downloading: {0}\n'.format(file_path))
        parent_dir = os.path.dirname(file_path)
        if parent_dir != '' and not os.path.exists(parent_dir):
            os.makedirs(parent_dir)

        try:
            response = self._item.http_session.get(self.url, stream=True)
            response.raise_for_status()
        except HTTPError as e:
            raise HTTPError('error downloading {0}, {1}'.format(self.url, e))
        with open(file_path, 'wb') as f:
            for chunk in response.iter_content(chunk_size=1024):
                if chunk:
                    f.write(chunk)
                    f.flush()
        log.info('downloaded {0}/{1} to {2}'.format(self.identifier,
                                                    self.name.encode('utf-8'),
                                                    file_path))

    # delete()
    # ____________________________________________________________________________________
    def delete(self, debug=False, verbose=False, cascade_delete=False, access_key=None,
               secret_key=None):
        """Delete a file from the Archive. Note: Some files -- such as
        <itemname>_meta.xml -- cannot be deleted.

        :type debug: bool
        :param debug: Set to True to print headers to stdout and exit
                      exit without sending the delete request.

        :type verbose: bool
        :param verbose: Print actions to stdout.

        :type cascade_delete: bool
        :param cascade_delete: Also deletes files derived from the file,
                               and files the file was derived from.
        """
        url = 'http://s3.us.archive.org/{0}/{1}'.format(self.identifier,
                                                        self.name.encode('utf-8'))
        access_key = self._item.session.access_key if not access_key else access_key
        secret_key = self._item.session.secret_key if not secret_key else secret_key
        request = iarequest.S3Request(
            method='DELETE',
            url=url,
            headers={'x-archive-cascade-delete': int(cascade_delete)},
            access_key=access_key,
            secret_key=secret_key
        )
        if debug:
            return request
        else:
            if verbose:
                msg = ' deleting: {0}'.format(self.name.encode('utf-8'))
                if cascade_delete:
                    msg += ' and all derivative files.\n'
                else:
                    msg += '\n'
                sys.stdout.write(msg)
            prepared_request = request.prepare()
            return self._item.http_session.send(prepared_request)



"""Upload files to Archive.org via the Internet Archive's S3 like server API.

IA-S3 Documentation: https://archive.org/help/abouts3.txt

usage:
    ia upload [--quiet] [--debug]
              (<identifier> <file>... | <identifier> - --remote-name=<name> | <identifier> <file> --remote-name=<name> | --spreadsheet=<metadata.csv>)
              [--metadata=<key:value>...] [--header=<key:value>...] [--checksum]
              [--no-derive] [--ignore-bucket] [--size-hint=<size>]
              [--delete] [--retries=<i>] [--sleep=<i>] [--log] [--no-collection-check]
    ia upload <identifier> --status-check
    ia upload --help

options:
    -h, --help
    -q, --quiet                       Turn off ia's output [default: False].
    -d, --debug                       Print S3 request parameters to stdout and
                                      exit without sending request.
    -r, --remote-name=<name>          When uploading data from stdin, this option
                                      sets the remote filename.
    -S, --spreadsheet=<metadata.csv>  bulk uploading...
    -m, --metadata=<key:value>...     Metadata to add to your item.
    -H, --header=<key:value>...       S3 HTTP headers to send with your request.
    -c, --checksum                    Skip based on checksum [default: False].
    -n, --no-derive                   Do not derive uploaded files.
    -i, --ignore-bucket               Destroy and respecify all metadata.
    -s, --size-hint=<size>            Specify a size-hint for your item.
    -R, --retries=<i>                 Number of times to retry request if S3
                                      retruns a 503 SlowDown error.
    -s, --sleep=<i>                   The amount of time to sleep between retries
                                      [default: 30].
    -l, --log                         Log upload results to file.
    --status-check                    Check if S3 is accepting requests to the
                                      given item.
    --delete                          Delete files after verifying checksums 
                                      [default: False].
    --no-collection-check             Skip checking if the collection being
                                      uploaded to exists [default: False].

"""
import sys
from tempfile import TemporaryFile
from xml.dom.minidom import parseString
from subprocess import call
import csv

from docopt import docopt, printable_usage
from requests.exceptions import HTTPError

from internetarchive.session import ArchiveSession
from internetarchive import get_item
from internetarchive.iacli.argparser import get_args_dict, get_xml_text


# _upload_files()
# ________________________________________________________________________________________
def _upload_files(args, identifier, local_file, upload_kwargs, prev_identifier=None,
                  archive_session=None):
    verbose = True if args['--quiet'] is False else False
    config = {} if not args['--log'] else {'logging': {'level': 'INFO'}}
    item = get_item(identifier, config=config)
    if args['--status-check']:
        if item.s3_is_overloaded():
            sys.stderr.write('warning: {0} is over limit, and not accepting requests. '
                             'Expect 503 SlowDown errors.\n'.format(identifier))
            sys.exit(1)
        else:
            sys.stdout.write('success: {0} is accepting requests.\n'.format(identifier))
            sys.exit(0)
    if (verbose) and (prev_identifier != identifier):
        sys.stdout.write('{0}:\n'.format(item.identifier))

    try:
        if isinstance(local_file, (list, tuple, set)) and args['--remote-name']:
            local_file = local_file[0]
        if args['--remote-name']:
            files = {args['--remote-name']: local_file}
        else:
            files = local_file
        response = item.upload(files, **upload_kwargs)
    except HTTPError as exc:
        response = [exc.response]
        if not response[0]:
            sys.exit(1)
        if response[0].status_code == 403:
            if (not item.session.access_key) and (not item.session.secret_key):
                sys.stderr.write('\nIAS3 Authentication failed. Please set your IAS3 '
                                 'access key and secret key \nvia the environment '
                                 'variables `IAS3_ACCESS_KEY` and `IAS3_SECRET_KEY`, '
                                 'or \nrun `ia configure` to add your IAS3 keys to your '
                                 'ia config file. You can \nobtain your IAS3 keys at the '
                                 'following URL:\n\n\t'
                                 'https://archive.org/account/s3.php\n\n')
            else:
                sys.stderr.write('\nIAS3 Authentication failed. It appears the keyset '
                                 '"{0}:{1}" \ndoes not have permission to upload '
                                 'to the given item or '
                                 'collection.\n\n'.format(item.session.access_key,
                                                          item.session.secret_key))
            sys.exit(1)

    if args['--debug']:
        for i, r in enumerate(response):
            if i != 0:
                sys.stdout.write('---\n')
            headers = '\n'.join(
                [' {0}: {1}'.format(k, v) for (k, v) in r.headers.items()]
            )
            sys.stdout.write('Endpoint:\n {0}\n\n'.format(r.url))
            sys.stdout.write('HTTP Headers:\n{0}\n'.format(headers))

    else:
        for resp in response:
            if not resp:
                continue
            if (resp.status_code == 200) or (not resp.status_code):
                continue
            error = parseString(resp.content)
            code = get_xml_text(error.getElementsByTagName('Code'))
            msg = get_xml_text(error.getElementsByTagName('Message'))
            sys.stderr.write(
                'error "{0}" ({1}): {2}\n'.format(code, resp.status_code, msg)
            )
            sys.exit(1)

# main()
# ________________________________________________________________________________________
def main(argv):
    args = docopt(__doc__, argv=argv)

    metadata = get_args_dict(args['--metadata'])
    # Make sure the collection being uploaded to exists.
    collection_id = metadata.get('collection')
    if collection_id and not args['--no-collection-check'] and not args['--status-check']:
        collection = get_item(collection_id) 
        if not collection.exists:
            sys.stderr.write(
                    'You must upload to a collection that exists. '
                    '"{0}" does not exist.\n{1}\n'.format(collection_id,
                                                          printable_usage(__doc__)))
            sys.exit(1)

    headers = get_args_dict(args['--header'])
    if args['--size-hint']:
        headers['x-archive-size-hint'] = args['--size-hint']

    # Upload keyword arguments.
    upload_kwargs = dict(
        metadata=metadata,
        headers=headers,
        debug=args['--debug'],
        queue_derive=True if args['--no-derive'] is False else False,
        ignore_preexisting_bucket=args['--ignore-bucket'],
        checksum=args['--checksum'],
        verbose=True if args['--quiet'] is False else False,
        retries=int(args['--retries']) if args['--retries'] else 0,
        retries_sleep=int(args['--sleep']),
        delete=args['--delete'],
    )

    if args['<file>'] == ['-'] and not args['-']:
        sys.stderr.write('--remote-name is required when uploading from stdin.\n')
        call(['ia', 'upload', '--help'])
        sys.exit(1)

    # Upload from stdin.
    if args['-']:
        local_file = TemporaryFile()
        local_file.write(sys.stdin.read())
        local_file.seek(0)
        _upload_files(args, args['<identifier>'], local_file, upload_kwargs)

    # Bulk upload using spreadsheet.
    elif args['--spreadsheet']:
        # Use the same session for each upload request.
        session = ArchiveSession()

        spreadsheet = csv.DictReader(open(args['--spreadsheet'], 'rU'))
        prev_identifier = None
        for row in spreadsheet:
            local_file = row['file']
            identifier = row['identifier']
            del row['file']
            del row['identifier']
            if (not identifier) and (prev_identifier):
                identifier = prev_identifier
            # TODO: Clean up how indexed metadata items are coerced
            # into metadata.
            md_args = ['{0}:{1}'.format(k.lower(), v) for (k, v) in row.items() if v]
            metadata = get_args_dict(md_args)
            upload_kwargs['metadata'].update(metadata)
            _upload_files(args, identifier, local_file, upload_kwargs, prev_identifier,
                          session)
            prev_identifier = identifier

    # Upload files.
    else:
        local_file = args['<file>']
        _upload_files(args, args['<identifier>'], local_file, upload_kwargs)



"""Download files from archive.org.

usage:
    ia download [--quiet] [--dry-run] [--ignore-existing] [--checksum]
                [--destdir=<dir>] [--no-directories] [--source=<source>... | --original]
                [--glob=<pattern> | --format=<format>...] [--concurrent] <identifier>
                [<file>...]
    ia download --help

options:
    -h, --help
    -q, --quiet               Turn off ia's output [default: False].
    -d, --dry-run             Print URLs to stdout and exit.
    -i, --ignore-existing     Clobber files already downloaded.
    -C, --checksum            Skip files based on checksum [default: False].
    -s, --source=<source>...  Only download files matching the given source.
    -o, --original            Only download files with source=original.
    -g, --glob=<pattern>      Only download files whose filename matches the
                              given glob pattern.
    -f, --format=<format>...  Only download files of the specified format(s).
                              You can use the following command to retrieve
                              a list of file formats contained within a given
                              item:

                                  ia metadata --formats <identifier>

    --no-directories          Download files into working directory. Do not
                              create item directories.
    --destdir=<dir>           The destination directory to download files
                              and item directories to.
    -c, --concurrent          Download files concurrently using the Python
                              gevent networking library (gevent must be
                              installed).

"""
import os
import sys

from docopt import docopt

from internetarchive import get_item


# ia_download()
#_________________________________________________________________________________________
def main(argv):
    args = docopt(__doc__, argv=argv)

    # Download specific files.
    if '/' in args['<identifier>']:
        identifier = args['<identifier>'].split('/')[0]
        files = [identifier.split('/')[1:]]
    else:
        identifier = args['<identifier>']
        files = args['<file>']

    item = get_item(identifier)
    if (args['--quiet'] is False) and (args['--dry-run'] is False):
        verbose = True
    else:
        verbose = False

    if files:
        if verbose:
            sys.stdout.write('{0}:\n'.format(identifier))
        for f in files:
            fname = f.encode('utf-8')
            if args['--no-directories']:
                path = fname
            else:
                path = os.path.join(identifier, fname)
            f = item.get_file(fname)
            if not f:
                sys.stderr.write(' {} doesn\'t exist!\n'.format(fname))
                continue
            if args['--dry-run']:
                sys.stdout.write(f.url + '\n')
            else:
                f.download(path, verbose, args['--ignore-existing'], args['--checksum'],
                           args['--destdir'])
        sys.exit(0)

    # Otherwise, download the entire item.
    if args['--source']:
        ia_source = args['--source']
    elif args['--original']:
        ia_source = ['original']
    else:
        ia_source = None

    item.download(
        concurrent=args['--concurrent'],
        source=ia_source,
        formats=args['--format'],
        glob_pattern=args['--glob'],
        dry_run=args['--dry-run'],
        verbose=verbose,
        ignore_existing=args['--ignore-existing'],
        checksum=args['--checksum'],
        destdir=args['--destdir'],
        no_directory=args['--no-directories'],
    )
    sys.exit(0)



"""Concurrently download metadata for items on Archive.org.

usage:
    ia mine [--cache | --output=<output.json>] [--workers=<count>] <itemlist.txt>
    ia mine --help

options:
    -h, --help
    -c, --cache                 Write item metadata to a file called <identifier>_meta.json
    -o, --output=<output.json>  Write all metadata to a single output file <itemlist>.json
    -w, --workers=<count>       The number of requests to run concurrently [default: 20]

"""
import sys
try:
    import ujson as json
except ImportError:
    import json

from docopt import docopt
from clint.textui import progress

from internetarchive import get_data_miner


# ia_mine()
#_________________________________________________________________________________________
def main(argv):
    args = docopt(__doc__, argv=argv)

    if args['<itemlist.txt>'] == '-':
        itemfile = sys.stdin
    else:
        itemfile = open(args['<itemlist.txt>'])
    with itemfile:
        identifiers = [i.strip() for i in itemfile]

    workers = int(args.get('--workers', 20)[0])
    miner = get_data_miner(identifiers, workers=workers)

    # Progress bar
    if args['--cache'] or args['--output']:
        if args['<itemlist.txt>'] != '-':
            itemfile_fname = args['<itemlist.txt>']
        else:
            itemfile_fname = 'stdin'
        miner = progress.bar(miner, expected_size=len(identifiers),
                             label='mining items from {0}: '.format(itemfile_fname))

    for i, item in miner:
        metadata = json.dumps(item._json)
        if args['--cache']:
            with open('{0}_meta.json'.format(item.identifier), 'w') as fp:
                fp.write(metadata)
        elif args['--output']:
            with open(args['--output'], 'a+') as fp:
                fp.write(metadata + '\n')
        else:
            try:
                sys.stdout.write(metadata + '\n')
            except IOError:
                break
    sys.exit(0)



try:
    from gevent import monkey, queue, spawn
    from gevent.hub import LoopExit
    monkey.patch_all(thread=False)
except ImportError:
    raise ImportError(
        """No module named gevent

        This feature requires the gevent neworking library. gevent
        and all of it's dependencies can be installed with pip:
        \tpip install cython gevent

        """)

from internetarchive import get_item
from internetarchive.session import get_session
from requests.exceptions import RequestException


# Mine class
# ________________________________________________________________________________________
class Mine(object):
    """This class is for concurrently retrieving metadata for items on
    Archive.org.

    Usage::

        >>> import internetarchive
        >>> miner = internetarchive.Mine(['identifier1', 'identifier2'], workers=50)
        >>> for md in miner:
        ...     print md

        """
    # __init__()
    # ____________________________________________________________________________________
    def __init__(self, identifiers, workers=20, max_requests=10):
        """Makes a generator for an list of `(index, item)` where `item`
        is an instance of `Item` containing metadata, and index is the index,
        for each id in `identifiers`. Note: this does not return the
        items in the same order as given in the identifiers list

        :type identifiers: list
        :param identifiers: a list of identifiers to get the metadata of
        :type workers: int
        :param workers: the number of concurrent workers to have fecthing the metadata
        :type max_requests: int or None
        :param max_requests: the number of times to try fetching the metadata,
        in case there is something wrong with requesting it

        :rtype: Mine

        """
        self.skips = []
        self.queue = queue
        self.workers = workers
        self.identifiers = identifiers
        self.item_count = len(identifiers)
        self.max_requests = max_requests
        self.queued_count = 0
        self.got_count = 0
        self.input_queue = self.queue.JoinableQueue(1000)
        self.json_queue = self.queue.Queue(1000)
        
        # Use the same session for each item fetch.
        self.session = get_session()

    # _metadata_getter()
    # ____________________________________________________________________________________
    def _metadata_getter(self):
        while True:
            i, identifier, num_requests = self.input_queue.get()
            try:
                item = get_item(identifier, archive_session=self.session)
                self.json_queue.put((i, item))
            except Exception as e:
                if (type(e) == RequestException and
                        (self.max_requests is None or num_requests < self.max_requests)):
                    self.input_queue.put((i, identifier, num_requests+1))
                else:
                    if identifier not in self.skips:
                        self.skips.append(identifier)
                    self.item_count -= 1
                    self.queued_count -= 1
                    if e.args is not None and len(e.args) > 0 and type(e.args[0]) == str:
                        e.args = ((e.args[0]+' when processing id '+repr(identifier),) +
                                  e.args[1:])
            finally:
                self.input_queue.task_done()

    # _queue_input()
    # ____________________________________________________________________________________
    def _queue_input(self):
        for i, identifier in enumerate(self.identifiers):
            if identifier not in self.skips:
                self.input_queue.put((i, identifier, 0))
                self.queued_count += 1

    # __iter__()
    # ____________________________________________________________________________________
    def __iter__(self):
        self.queued_count = 0
        self.got_count = 0
        spawn(self._queue_input)
        for i in range(self.workers):
            spawn(self._metadata_getter)

        def metadata_iterator_helper():
            while self.queued_count < self.item_count or self.got_count < self.queued_count:
                self.got_count += 1
                try:
                    yield self.json_queue.get()
                except LoopExit:
                    raise StopIteration

        return metadata_iterator_helper()



from setuptools import setup
import sys
import ast
import re
from codecs import open


_version_re = re.compile(r'__version__\s+=\s+(.*)')
with open('internetarchive/__init__.py', 'r', 'utf-8') as f:
    version = str(ast.literal_eval(_version_re.search(f.read()).group(1)))

with open('README.rst', 'r', 'utf-8') as f:
    readme = f.read()
with open('HISTORY.rst', 'r', 'utf-8') as f:
    history = f.read()

setup(
    name='internetarchive',
    version=version,
    author='Jacob M. Johnson',
    author_email='jake@archive.org',
    packages=['internetarchive', 'internetarchive.iacli'],
    entry_points = {
        'console_scripts': [
            'ia = internetarchive.iacli.ia:main',
        ],
    },
    url='https://github.com/jjjake/ia-wrapper',
    license='AGPL 3',
    description='A python interface to archive.org.',
    long_description=readme + '\n\n' + history,
    install_requires=[
        'requests==2.7.0',
        'jsonpatch==0.4',
        'docopt==0.6.2',
        'PyYAML==3.11',
        'clint==0.3.3',
        'six==1.4.1',
    ],
    extras_require = {
        'speedups': [
            'ujson==1.33',
            'Cython==0.18',
            'gevent==1.0',
        ],
    },
    classifiers=(
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
    )
)



"""Retrieve and modify metadata for items on archive.org.

usage:
    ia metadata [--modify=<key:value>...] [--target=<target>] [--priority=<priority>] <identifier>...
    ia metadata [--spreadsheet=<metadata.csv>] [--priority=<priority>] [--modify=<key:value>...]
    ia metadata [--append=<key:value>...] [--priority=<priority>] <identifier>...
    ia metadata [--exists | --formats] <identifier>...
    ia metadata --help

options:
    -h, --help
    -m, --modify=<key:value>          Modify the metadata of an item.
    -t, --target=<target>             The metadata target to modify.
    -a, --append=<key:value>          Append metadata to an element.
    -s, --spreadsheet=<metadata.csv>  Modify metadata in bulk using a spreadsheet as input.
    -e, --exists                      Check if an item exists
    -F, --formats                     Return the file-formats the given item contains.
    -p, --priority=<priority>        Set the task priority.

"""
import sys
try:
    import ujson as json
except ImportError:
    import json
import csv

from docopt import docopt

from internetarchive import get_item
from internetarchive.iacli.argparser import get_args_dict


# modify_metadata()
# ________________________________________________________________________________________
def modify_metadata(item, metadata, args):
    append = True if args['--append'] else False
    r = item.modify_metadata(metadata, target=args['--target'], append=append,
                             priority=args['--priority'])
    if not r.json()['success']:
        error_msg = r.json()['error']
        sys.stderr.write(u'{0} - error ({1}): {2}\n'.format(item.identifier, r.status_code,
                                                           error_msg))
        return r
    sys.stdout.write('{0} - success: {1}\n'.format(item.identifier,
                                                   r.json()['log']))
    return r


# main()
# ________________________________________________________________________________________
def main(argv):
    args = docopt(__doc__, argv=argv)

    formats = set()
    responses = []

    for i, _item in enumerate(args['<identifier>']):
        item = get_item(_item)

        # Check existence of item.
        if args['--exists']:
            if item.exists:
                responses.append(True)
                sys.stdout.write('{0} exists\n'.format(item.identifier))
            else:
                responses.append(False)
                sys.stderr.write('{0} does not exist\n'.format(item.identifier))
            if (i + 1) == len(args['<identifier>']):
                if all(r is True for r in responses):
                    sys.exit(0)
                else:
                    sys.exit(1)

        # Modify metadata.
        elif args['--modify'] or args['--append']:
            metadata_args = args['--modify'] if args['--modify'] else args['--append']
            metadata = get_args_dict(metadata_args)
            responses.append(modify_metadata(item, metadata, args))
            if (i + 1) == len(args['<identifier>']):
                if all(r.status_code == 200 for r in responses):
                    sys.exit(0)
                else:
                    sys.exit(1)

        # Get metadata.
        elif args['--formats']:
            for f in item.iter_files():
                formats.add(f.format)
            if (i + 1) == len(args['<identifier>']):
                sys.stdout.write('\n'.join(formats) + '\n')

        # Dump JSON to stdout.
        else:
            metadata = json.dumps(item._json)
            sys.stdout.write(metadata + '\n')

    # Edit metadata for items in bulk, using a spreadsheet as input.
    if args['--spreadsheet']:
        if not args['--priority']:
            args['--priority'] = -5
        spreadsheet = csv.DictReader(open(args['--spreadsheet'], 'rU'))
        responses = []
        for row in spreadsheet:
            if not row['identifier']:
                continue
            item = get_item(row['identifier'])
            if row.get('file'):
                del row['file']
            metadata = dict((k.lower(), v) for (k, v) in row.items() if v)
            responses.append(modify_metadata(item, metadata, args))

        if all(r.status_code == 200 for r in responses):
            sys.exit(0)
        else:
            sys.exit(1)

    sys.exit(0)



try:
    import ujson as json
except ImportError:
    import json
from six.moves.urllib.parse import parse_qsl

import requests.sessions

from . import session


# Catalog class
# ________________________________________________________________________________________
class Catalog(object):
    """:todo: Document Catalog Class."""

    ROW_TYPES = dict(
        green=0,
        blue=1,
        red=2,
        brown=9,
        purple=-1,
    )

    # init()
    # ____________________________________________________________________________________
    def __init__(self, identifier=None, task_ids=None, params={}, verbose=True,
                 config=None):
        verbose = '1' if verbose else '0'
        params = {} if not params else params

        self.session = session.ArchiveSession(config)
        # Accessing the Archive.org catalog requires a users
        # logged-in-* cookies (i.e. you must be logged in).
        # Raise an exception if they are not set.
        if not self.session.cookies.get('logged-in-user'):
            raise NameError('logged-in-user cookie not set. Use `ia configure --cookies` '
                            'to add your logged-in-user cookie to your internetarchive '
                            'config file, or set the IA_LOGGED_IN_USER environment '
                            'variable.')
        elif not self.session.cookies.get('logged-in-sig'):
            raise NameError('logged-in-sig cookie not set. Use `ia configure --cookies` '
                            'to add your logged-in-sig cookie to your internetarchive '
                            'config file, or set the IA_LOGGED_IN_SIG environment '
                            'variable.')

        self.http_session = requests.sessions.Session()

        # Set cookies from config.
        self.http_session.cookies = self.session.cookies
        self.http_session.cookies['verbose'] = verbose

        # Params required to retrieve JSONP from the IA catalog.
        self.params = dict(
            json=2,
            output='json',
            callback='foo',
        )
        self.params.update(params)
        # Return user's current tasks as default.
        if not identifier and not task_ids and not params:
            self.params['justme'] = 1

        if task_ids:
            if not isinstance(task_ids, (set, list)):
                task_ids = [task_ids]
            self.params.update(dict(
                where='task_id in({tasks})'.format(tasks=','.join(task_ids)),
                history=99999999999999999999999,  # TODO: is there a better way?
            ))

        if identifier:
            self.url = 'http://archive.org/history/{id}'.format(id=identifier)
        elif task_ids:
            self.url = 'http://cat-tracey.archive.org/catalog.php'
        else:
            self.url = 'http://archive.org/catalog.php'

        # Get tasks.
        self.tasks = self._get_tasks()

        # Set row_type attrs.
        for key in self.ROW_TYPES:
            rows = [t for t in self.tasks if t.row_type == self.ROW_TYPES[key]]
            setattr(self, '{0}_rows'.format(key), rows)

    # _get_tasks()
    # ____________________________________________________________________________________
    def _get_tasks(self):
        r = self.http_session.get(self.url, params=self.params)
        # Convert JSONP to JSON (then parse the JSON).
        json_str = r.content[(r.content.index("(") + 1):r.content.rindex(")")]
        return [
            CatalogTask(t, http_session=self.http_session) for t in json.loads(json_str)
        ]


# CatalogTask class
# ________________________________________________________________________________________
class CatalogTask(object):
    """
    Represents catalog task.

    """

    COLUMNS = (
        'identifier',
        'server',
        'command',
        'time',
        'submitter',
        'args',
        'task_id',
        'row_type'
    )

    # init()
    # ____________________________________________________________________________________
    def __init__(self, columns, http_session=None):
        if not http_session:
            self._http_session = requests.sessions.Session()
        else:
            self._http_session = http_session

        for key, value in map(None, self.COLUMNS, columns):
            if key:
                setattr(self, key, value)
        # special handling for 'args' - parse it into a dict if it is a string
        if isinstance(self.args, basestring):
            self.args = dict(x for x in parse_qsl(self.args.encode('utf-8')))

    # __repr__()
    # ____________________________________________________________________________________
    def __repr__(self):
        return ('CatalogTask(identifier={identifier},'
                ' task_id={task_id!r}, server={server!r},'
                ' command={command!r},'
                ' submitter={submitter!r},'
                ' row_type={row_type})'.format(**self.__dict__))

    # __getitem__()
    # ____________________________________________________________________________________
    def __getitem__(self, key):
        """
        Dict-like access provided as backward compatibility.

        """
        if key in self.COLUMNS:
            return getattr(self, key, None)
        else:
            raise KeyError(key)

    # task_log()
    # ____________________________________________________________________________________
    def task_log(self):
        """
        Return file-like reading task log.

        """
        if self.task_id is None:
            raise ValueError('task_id is None')
        url = 'http://catalogd.archive.org/log/{0}'.format(self.task_id)
        p = dict(full=1)
        r = self._http_session.get(url, params=p)
        r.raise_for_status()
        return r.content



"""Delete files from Archive.org via the Internet Archive's S3 like server API.

IA-S3 Documentation: https://archive.org/help/abouts3.txt

usage:
    ia delete [--quiet] [--debug] [--dry-run] [--cascade] <identifier> <file>...
    ia delete [--quiet] [--debug] [--dry-run] --all <identifier>
    ia delete [--quiet] [--debug] [--dry-run] --glob=<pattern> <identifier>
    ia delete --help

options:
    -h, --help
    -q, --quiet            Print status to stdout. 
    -c, --cascade          Delete all derivative files associated with the given file.
    -a, --all              Delete all files in the given item (Note: Some files, such
                           as <identifier>_meta.xml and <identifier>_files.xml, cannot
                           be deleted)
    -d, --dry-run          Output files to be deleted to stdout, but don't actually delete.
    -g, --glob=<pattern>   Only return patterns match the given pattern.

"""
import sys
from xml.dom.minidom import parseString
from fnmatch import fnmatch

from docopt import docopt

from internetarchive import get_item
from internetarchive.iacli.argparser import get_xml_text


# main()
#_________________________________________________________________________________________
def main(argv):
    args = docopt(__doc__, argv=argv)
    verbose = True if not args['--quiet'] else False
    item = get_item(args['<identifier>'])

    # Files that cannot be deleted via S3.
    no_delete = ['_meta.xml', '_files.xml', '_meta.sqlite']

    if verbose:
        sys.stdout.write('Deleting files from {0}\n'.format(item.identifier))

    if args['--all']:
        files = [f for f in item.iter_files()]
        args['--cacade'] = True
    elif args['--glob']:
        files = item.get_files(glob_pattern=args['--glob'])
    else:
        fnames = []
        if args['<file>'] == ['-']:
            fnames = [f.strip().decode('utf-8') for f in sys.stdin]
        else:
            fnames = [f.strip().decode('utf-8') for f in args['<file>']]

        files = [f for f in [item.get_file(f) for f in fnames] if f]

    if not files:
        sys.stderr.write(' warning: no files found, nothing deleted.\n')
        sys.exit(1)

    for f in files:
        if not f:
            if verbose:
                sys.stderr.write(' error: "{0}" does not exist\n'.format(f.name))
            sys.exit(1)
        if any(f.name.endswith(s) for s in no_delete):
            continue
        if args['--dry-run']:
            sys.stdout.write(' will delete: {0}/{1}\n'.format(item.identifier, 
                                                             f.name.encode('utf-8')))
            continue
        resp = f.delete(verbose=verbose, cascade_delete=args['--cascade'])
        if resp.status_code != 204:
            error = parseString(resp.content)
            msg = get_xml_text(error.getElementsByTagName('Message'))
            sys.stderr.write(' error: {0} ({1})\n'.format(msg, resp.status_code))
            sys.exit(1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Test uploading through the internetarchive python package.

This test script creates a new archive.org item, and therefore is named in
such a way that py.test does not automatically run it.
"""

import internetarchive as ia
import datetime
import hashlib
import os
import StringIO
import string
import tempfile
import time

import internetarchive.config as ic


s3_conf = ic.get_config().get('s3', {})
access = s3_conf.get('access_key')
secret = s3_conf.get('secret_key')


# get_new_item()
#_________________________________________________________________________________________
def get_new_item():
    """return an ia item object for an item that does not yet exist"""
    now = datetime.datetime.utcnow()
    item_name = 'test_upload_iawrapper_' + now.strftime('%Y_%m_%d_%H%M%S')
    item = ia.Item(item_name)
    if item.exists is False:
        return item

    raise KeyError, 'Could not find a unique item name after 5 tries'


# get_file()
#_________________________________________________________________________________________
def get_file(item_name, file_name):
    """get a file from a newly-created item. Wait for file to land in item, retry if needed"""

    for i in range(5):
        print '  waiting 30 seconds for upload of', file_name
        time.sleep(30)

        item = ia.Item(item_name)
        f = item.get_file(file_name)
        if f is not None:
            return f

    raise KeyError, 'Could not retrieve file after 5 tries'


# upload_stringIO()
#_________________________________________________________________________________________
def upload_stringIO(item, metadata=dict(collection='test_collection')):
    contents = 'hello world'
    name = 'hello_world.txt'
    fh = StringIO.StringIO(contents)
    fh.name = name

    r = item.upload(fh, metadata=metadata, access_key=access,
                    secret_key=secret)

    f = get_file(item.identifier, name)
    assert f.sha1 == hashlib.sha1(contents).hexdigest()


# upload_tempfile()
#_________________________________________________________________________________________
def upload_tempfile(item):
    contents = 'temporary file contents'
    temp_file = tempfile.NamedTemporaryFile(suffix='.txt')

    temp_file.write(contents)
    temp_file.seek(0, os.SEEK_SET)

    item.upload(temp_file.name, metadata= {
            'collection': 'test_collection',
            'description': 'ℛℯα∂α♭ℓℯ ♭ʊ☂ η☺т Ѧ$☾ℐℐ, ¡ooʇ ןnɟǝsn sı uʍop-ǝpısdn' + string.whitespace,
        }, access_key=access, secret_key=secret)
    temp_file.close()

    f = get_file(item.identifier, os.path.basename(temp_file.name))
    assert f.sha1 == hashlib.sha1(contents).hexdigest()


# upload_two_new_tems()
#_________________________________________________________________________________________
def upload_two_new_items():
    first_item = get_new_item()
    second_item = get_new_item()

    upload_stringIO(first_item, metadata={
            'collection': 'test_collection',
            'description': 'testing ia-wrapper {}'.format(first_item.identifier),
        })
    upload_stringIO(second_item, metadata={
            'collection': 'test_collection',
            'description': '',
        })
    first_item.get_metadata()
    second_item.get_metadata()
    assert first_item.metadata['description'] != second_item.metadata['description']


# test_upload()
#_________________________________________________________________________________________
def test_upload():
    print 'Finding new item name'
    item = get_new_item()

    print 'Uploading new item named', item.identifier

    print 'Testing upload using StringIO'
    upload_stringIO(item)

    print 'Testing upload using tempfile'
    upload_tempfile(item)

    print 'Testing that subsequent uploads do not have headers from previous uploads'
    upload_two_new_items()

    print 'Finished upload test'


# main()
#_________________________________________________________________________________________
if __name__ == '__main__':
    if os.environ.get('IAS3_ACCESS_KEY') is None:
        raise LookupError, 'You must set IAS3_ACCESS_KEY environment variable!'

    if os.environ.get('IAS3_SECRET_KEY') is None:
        raise LookupError, 'You must set IAS3_SECRET_KEY environment variable!'

    test_upload()



import os, sys
from subprocess import Popen, PIPE
from time import time

import pytest

inc_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0, inc_path)


import internetarchive.config
try:
    import internetarchive.mine
    py_test_mine = True
except ImportError:
    py_test_mine = False


@pytest.mark.skipif('py_test_mine == False', reason='requires internetarchive[speedups]')
def test_ia_mine():
    with open('testlist.txt', 'w') as fp:
        fp.write('\n'.join(['nasa', 'iacli-test-item']))

    cmd = 'ia mine testlist.txt --cache'
    proc = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE)
    stdout, stderr = proc.communicate()
    assert proc.returncode == 0

    cmd = 'ia mine testlist.txt --output=d.json'
    proc = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE)
    stdout, stderr = proc.communicate()
    assert proc.returncode == 0

    cmd = 'ia mine testlist.txt'
    proc = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE)
    stdout, stderr = proc.communicate()
    assert proc.returncode == 0

    # Test ids from stdin.
    cmd = 'echo "nasa" | ia mine -'
    proc = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE)
    stdout, stderr = proc.communicate()
    assert proc.returncode == 0

    try:
        os.remove('testlist.txt')
        os.remove('d.json')
        os.remove('nasa_meta.json')
        os.remove('iacli-test-item_meta.json')
    except OSError:
        pass
